import { createRouter, createWebHashHistory, Router, RouteRecordRaw } from 'vue-router'
import Layout from '@/components/Main.vue'
import asyncBaseRoutes from "@/views/base/router"

const routes: Array<RouteRecordRaw> = [
  {
    path: '/login',
    name: 'login',
    component: () => import('@/views/base/views/login/index.vue')
  }
]
// 动态路由，登录后根据返回的菜单动态注册账号拥有的权限路由
export const asyncRoutes: Array<RouteRecordRaw> = [
 ...asyncBaseRoutes,
]

// 404路由，需动态注册到末尾
export const notFoundRoutes: Array<RouteRecordRaw> = [
  {
    path: '/:pathMatch(.*)*', // 404 页面
    redirect: '/login',
    meta: {
      isAsync: true
    },
    component: () => import('@/views/base/views/login/index.vue')
  }
]
const router: Router = createRouter({
  history: createWebHashHistory(),
  routes
})

export function resetRouter() {
  // 重置路由
  // 注意：所有动态路由路由必须带有 name 属性，否则可能会不能完全重置干净
  try {
    router.getRoutes().forEach((route) => {
      const { name, meta } = route
      // 只清除动态路由:由meta.isAsync设置
      if (name && meta.isAsync && router.hasRoute(name)) {
        router.removeRoute(name)
      }
    })
  } catch (error) {
    // 强制刷新浏览器，不过体验不是很好
    window.location.reload()
  }
}

export default router
