import NProgress from 'nprogress'
import 'nprogress/nprogress.css'
import { RouteLocationNormalized } from 'vue-router'
// import { ElMessage } from 'element-plus'
import router from '@/router'
import { store } from '@/store'
import { getToken } from '@/utils/auth'
import whiteList from './white-list'

NProgress.configure({ showSpinner: false })

router.beforeEach(
  async (to: RouteLocationNormalized, _: RouteLocationNormalized, next: any) => {
    NProgress.start()
    // 判断该用户是否登录
    if (getToken()) {
      // 若后续'/'匹配某些功能，则去掉下面跳转配置
      if (to.path === '/') {
        next('/login')
        NProgress.done()
        return
      }
      // 刷新后store中menu和动态route丢失问题
      if (to.path !== '/login') {
        let { menuData } = store.state.user
        if ((!menuData || menuData.length < 1) && sessionStorage.getItem('menuData')) {
          menuData = JSON.parse(sessionStorage.getItem('menuData') || '[]')
          await store.dispatch('user/setMenuRoute', menuData)
          // 重新注册后需要重新进行跳转，否则不生效
          next(to.path)
          NProgress.done()
        } else {
          next()
        }
      } else {
        next()
      }
    } else if (whiteList.indexOf(to.path) !== -1) {
      // 如果在免登录的白名单中，则直接进入
      next()
    } else {
      // 其他没有访问权限的页面将被重定向到登录页面
      next('/login')
      NProgress.done()
    }
  }
)

router.afterEach(() => {
  NProgress.done()
})
