import { Module } from 'vuex'
import RootStateTypes from '../../types'
import IDialogState from './types'

const dialog: Module<IDialogState, RootStateTypes> = {
  namespaced: true,
  state: {
    dialogVisible: false,
    params: null,
    needReload: false, // 是否需要刷新页面(新增或修改后置为true)
    isRouterAlive: true
  },
  mutations: {
    open(state, params) {
      state.params = params
      state.dialogVisible = true
      state.needReload = false
    },
    close(state, needReload = false) {
      state.needReload = needReload
      state.dialogVisible = false
    },
    // 重置params
    CLEAR_PARAMS(state) {
      state.params = null
    },
    routerTrue(state) {
      state.isRouterAlive = true
    },
    routerFalse(state) {
      state.isRouterAlive = false
    }
  },
  actions: {
    openDialog(state, params) {
      state.commit('open', params)
    },
    closeDialog({ state, commit }, needReload) {
      // 窗口是打开状态再commit('close')(触发computed的set函数进入时防止needReload丢失)
      if (state.dialogVisible) {
        commit('close', needReload)
      }
    },
    resetParams({ commit }) {
      commit('CLEAR_PARAMS')
    },
    setRouterAlive(state, status) {
      if (status) {
        state.commit('routerTrue')
      } else {
        state.commit('routerFalse')
      }
    }
  }
}

export default dialog
