import { Module } from 'vuex'
import RootStateTypes from '../../types'
import IDicState from './types'
import Common from '@/views/base/api/common'
import AssetManage from '@/views/base/api/assetManage'
import DicType from '@/utils/dicType'

const dicInfo: Module<IDicState, RootStateTypes> = {
  namespaced: true,
  state: {
    getDicInfoByDicType: null, // 链路类型
    bizOperateMode: null, // 业务操作方式
    systemType: null, // 业务操作方式
    taskStatus: null, // 任务状态
    dataExchange: null, // 数据交换方式
    protocolCode: null, // 协议代码
    systemLogType: null // 系统日志-日志类型
  },
  mutations: {
    setDicData(state, params) {
      state[params.dicType] = params.data
    }
  },
  actions: {
    // 根据数据字典类型加载数据字典
    getDicInfoByDicType({ state, commit }, typeName: string) {
      return new Promise((resolve, reject) => {
        if (state[typeName]) {
          resolve(state[typeName])
        } else {
          Common.getDicInfo(typeName)
            .then((res: any) => {
              commit('setDicData', {
                data: res.data.data,
                dicType: typeName
              })
              resolve(res.data.data)
            })
            .catch(() => {
              reject()
            })
        }
      })
    },
    // 加载链路类型数据字典
    getLinkType({ state, commit }) {
      return new Promise((resolve, reject) => {
        if (state[DicType.connObjList]) {
          resolve(state[DicType.connObjList])
        } else {
          Common.getLinkType()
            .then((res: any) => {
              commit('setDicData', {
                data: res.data.data,
                dicType: 'connObjList'
              })
              resolve(res.data.data)
            })
            .catch(() => {
              reject()
            })
        }
      })
    },
    // 加载协议代码数据字典
    getProtocol({ state, commit }) {
      return new Promise((resolve, reject) => {
        if (state[DicType.protocolCode]) {
          resolve(state[DicType.protocolCode])
        } else {
          AssetManage.getProtocol({})
            .then((res: any) => {
              commit('setDicData', {
                data: res.data.data,
                dicType: 'protocolCode'
              })
              resolve(res.data.data)
            })
            .catch(() => {
              reject()
            })
        }
      })
    }
  }
}

export default dicInfo
