import { Module } from 'vuex'
import { RouteRecordRaw } from 'vue-router'
import RootStateTypes from '../../types'
import IUserState from './types'
import { getToken, removeToken } from '@/utils/auth'
import router, { asyncRoutes, notFoundRoutes, resetRouter } from '@/router'


interface menuItem {
  id: string | number
  name: string
  icon?: string
  url: string
  [propName: string]: any
}

// 根据路由名称找到对应路由对象
function findRouterByUrl(name: string, routes?: any) {
  return (routes || asyncRoutes).find((item: RouteRecordRaw) => item.path === name)
}

const userModule: Module<IUserState, RootStateTypes> = {
  namespaced: true,
  state: {
    token: getToken() || '',
    name: '',
    menuData: [],
    params: {} // 主要存类似路由传参但刷新要缓存的数据
  },
  mutations: {
    SET_TOKEN: (state: IUserState, token: string) => {
      state.token = token
    },
    SET_PARAMS: (state: IUserState, params: object) => {
      state.params = params
    },
    EXTEND_PARAMS: (state: IUserState, fieldObj: object) => {
      state.params = {
        ...state.params,
        ...fieldObj
      }
    },
    SET_MENU: (state: IUserState, menus: object[]) => {
      state.menuData = menus
    }
  },
  actions: {
    // // 登录
    // login({ commit }: any, userInfo: { username: string; password: string }) {
    //   return new Promise((resolve, reject) => {
    //     // console.log('userInfo:', userInfo)
    //     /* try {
    //       const res = {
    //         data: {
    //           accessToken: 'accessToken'
    //         }
    //       }
    //       setToken(res.data.accessToken)
    //       commit('SET_TOKEN', res.data.accessToken)
    //       resolve(true)
    //     } catch {
    //       reject()
    //     } */
    //     accountLogin(userInfo)
    //       .then((res: any) => {
    //         setToken(res.data.accessToken)
    //         commit('SET_TOKEN', res.data.accessToken)
    //         resolve(true)
    //       })
    //       .catch((error) => {
    //         reject(error)
    //       })
    //   })
    // },
    // 根据菜单动态设置路由
    setMenuRoute({ commit }: any, menuData: any) {
      commit('SET_MENU', menuData)
      return new Promise((resolve) => {
        // 清空之前注册
        resetRouter()
        // 动态注册路由 asyncRoutes
        let roleRouter: Array<RouteRecordRaw> = []
        menuData.forEach((item: menuItem) => {
          const r = findRouterByUrl(item.url)
          if (r) {
            const registerRouter = {
              path: r.path,
              name: r.name,
              component: r.component,
              meta: r.meta,
              children: []
            }
            // 有重定向且只有一个子路由，直接进行相应赋值
            if (r.redirect && r.children.length === 1) {
              registerRouter.redirect = r.redirect
              registerRouter.children = r.children
            }
            // 有子菜单，需要注册子菜单
            if (item.children) {
              item.children.forEach((subitem: menuItem) => {
                const subr = findRouterByUrl(subitem.url, r.children || [])
                if (subr) {
                  registerRouter.children.push({
                    path: subr.path,
                    name: subr.name,
                    component: subr.component,
                    meta: subr.meta,
                    children: subr.children // 三级router
                  })
                }
              })
            }
            roleRouter.push(registerRouter)
          }
        })
        roleRouter = roleRouter.concat(notFoundRoutes)
        roleRouter.forEach((route) => {
          router.addRoute(route)
        })
        resolve(true)
      })
    },
    setParams({ commit }: any, params: any) {
      commit('SET_PARAMS', params)
    },
    extendParams({ commit }: any, fieldObj: any) {
      commit('EXTEND_PARAMS', fieldObj)
    },
    // 登出
    logout({ commit }: any) {
      removeToken()
      commit('SET_TOKEN', '')
    },
    // 重置 token
    resetToken({ commit }: any) {
      removeToken()
      commit('SET_TOKEN', '')
    }
  }
}

export default userModule
