/**
 * echarts自适应相关
 */
import { ref } from 'vue'

export default () => {
  const chart = ref<any>()

  // 节流(频繁触发后立即调用一次，wait内只调用一次，结束的时候再调用一次)
  function throttle(fn: any, wait?: number) {
    const time = wait || 400
    let cur = 0
    let timer: any = null
    return (e: any) => {
      const now = new Date().getTime()
      if (now - cur > time) {
        fn(e)
        cur = now
      } else if (timer) {
        clearTimeout(timer)
      }
      timer = setTimeout(() => {
        fn(e)
      }, time)
    }
  }

  // 调用chart的resize方法
  const chartResizeHandler = throttle(() => {
    if (chart.value) {
      chart.value.resize()
    }
  }, 100)

  // 初始化resize事件
  const initResizeEvent = () => {
    window.addEventListener('resize', chartResizeHandler)
  }
  // 移除resize事件
  const destroyResizeEvent = () => {
    window.removeEventListener('resize', chartResizeHandler)
  }

  // 页面挂载完成时
  const mountedHandle = () => {
    initResizeEvent()
  }
  // keep-alive页面活动时
  const activatedHandle = () => {
    initResizeEvent()
  }
  // 页面销毁前
  const beforeDestroyHandle = () => {
    destroyResizeEvent()
  }
  // keep-alive页面失活时
  const deactivatedHandle = () => {
    destroyResizeEvent()
  }

  return {
    chart,
    mountedHandle,
    activatedHandle,
    beforeDestroyHandle,
    deactivatedHandle
  }
}
