export default class Rules {
  /*
   * ip验证
   * */
  public static IP(rule: any, value: any, callback: any): void {
    if (value === '') {
      if (callback) callback(new Error('请输入正确IP地址'))
    }
    if (Rules.IPV4('', value) || Rules.IPV6('', value)) {
      if (callback) callback()
    } else if (callback) callback(new Error('IP地址不正确'))
  }

  /*
   * 子网掩码验证
   * */
  public static netMask(rule: any, value: any, callback: any): void {
    const reg =
      /(254|252|248|240|224|192|128|0)\.0\.0\.0|255\.(254|252|248|240|224|192|128|0)\.0\.0|255\.255\.(254|252|248|240|224|192|128|0)\.0|255\.255\.255\.(254|252|248|240|224|192|128|0)/
    if (!value) {
      return callback(new Error('请输入正确的子网掩码'))
    }
    if (!reg.test(value)) {
      return callback(new Error('请输入正确的子网掩码'))
    }
    return callback()
  }

  /*
   * 网关验证
   * */
  public static gateWay(rule: any, value: any, callback: any): void {
    const reg = /[0-9].[0-9].[0-9].[0-9]/
    if (!value) {
      return callback(new Error('请输入正确的网关'))
    }
    if (!reg.test(value)) {
      return callback(new Error('请输入正确的网关'))
    }
    return callback()
  }

  /*
   * DNS验证
   * */
  public static dns(rule: any, value: any, callback: any): void {
    if (value === '') {
      if (callback) callback(new Error('请输入正确的DNS'))
    }
    if (Rules.IPV4('', value) || Rules.IPV6('', value)) {
      if (callback) callback()
    } else if (callback) callback(new Error('请输入正确的DNS'))
  }

  /*
   * ipv4验证
   * */
  public static IPV4(rule: any, value: any, callback?: any) {
    if (value === '') {
      if (callback) callback(new Error('请输入正确IP地址'))
      return false
    }
    let ok = true
    const ips = value.split(',')

    const ip =
      /^((\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5]))(\/(\d{1,2}|1\d\d|2[0-4]\d|25[0-5]))?$/
    for (let i: number = 0; i < ips.length; i += 1) {
      if (!ip.test(ips[i])) {
        ok = false
        break
      }
    }
    if (ok) {
      if (callback) callback()
    } else if (callback) callback(new Error('IP地址格式错误'))
    return ok
  }

  /*
   * ipv6验证
   * */
  public static IPV6(rule: any, value: any, callback?: any) {
    if (value === '') {
      if (callback) callback(new Error('请输入正确IP地址'))
      return false
    }

    const ipv6 =
      /^([\da-fA-F]{1,4}:){6}((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$|^::([\da-fA-F]{1,4}:){0,4}((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$|^([\da-fA-F]{1,4}:):([\da-fA-F]{1,4}:){0,3}((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$|^([\da-fA-F]{1,4}:){2}:([\da-fA-F]{1,4}:){0,2}((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$|^([\da-fA-F]{1,4}:){3}:([\da-fA-F]{1,4}:){0,1}((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$|^([\da-fA-F]{1,4}:){4}:((25[0-5]|2[0-4]\d|[01]?\d\d?)\.){3}(25[0-5]|2[0-4]\d|[01]?\d\d?)$|^([\da-fA-F]{1,4}:){7}[\da-fA-F]{1,4}$|^:((:[\da-fA-F]{1,4}){1,6}|:)$|^[\da-fA-F]{1,4}:((:[\da-fA-F]{1,4}){1,5}|:)$|^([\da-fA-F]{1,4}:){2}((:[\da-fA-F]{1,4}){1,4}|:)$|^([\da-fA-F]{1,4}:){3}((:[\da-fA-F]{1,4}){1,3}|:)$|^([\da-fA-F]{1,4}:){4}((:[\da-fA-F]{1,4}){1,2}|:)$|^([\da-fA-F]{1,4}:){5}:([\da-fA-F]{1,4})?$|^([\da-fA-F]{1,4}:){6}:$/
    if (ipv6.test(value)) {
      if (callback) callback()
      return true
    }
    if (callback) callback(new Error('IP地址格式错误'))
    return false
  }

  /*
   * 服务器地址验证
   * */
  public static serverAdress(rule: any, value: any, callback: any) {
    if (value === '') {
      callback(new Error('请输入正确服务器地址'))
    }
    // const ip =
    //   /^((\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5]))(\/(\d{1,2}|1\d\d|2[0-4]\d|25[0-5]))?$/
    //   var email =/^[A-Za-z0-9\u4e00-\u9fa5]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$/
    // const strRegex =
    //   '^((https|http|ftp|rtsp|mms)?://)' +
    //   "?(([0-9a-z_!~*'().&=+$%-]+: )?[0-9a-z_!~*'().&=+$%-]+@)?" + // ftp的user@
    //   '(([0-9]{1,3}.){3}[0-9]{1,3}' + // IP形式的URL- 199.194.52.184
    //   '|' + // 允许IP和DOMAIN（域名）
    //   "([0-9a-z_!~*'()-]+.)*" + // 域名- www.
    //   '([0-9a-z][0-9a-z-]{0,61})?[0-9a-z].' + // 二级域名
    //   '[a-z]{2,6})' + // first level domain- .com or .museum
    //   '(:[0-9]{1,4})?' + // 端口- :80
    //   '((/?)|' + // a slash isn't required if there is no file name
    //   "(/[0-9a-z_!~*'().;?:@&=+$,%#-]+)+/?)$"

    // const re = new RegExp(ip)
    const re1 =
      /^((https|http|ftp|rtsp|mms)?:\/\/)?(([0-9a-z_!~*'().&=+$%-]+: )?[0-9a-z_!~*'().&=+$%-]+@)?(([0-9]{1,3}.){3}[0-9]{1,3}|([0-9a-z_!~*'()-]+.){0,5}([0-9a-z][0-9a-z-]{0,61})?[0-9a-z].[a-z]{2,6})$/
    // const re1 = new RegExp(strRegex)
    if (Rules.IPV4('', value) || Rules.IPV6('', value) || re1.test(value)) {
      callback()
    } else {
      callback(new Error('请输入正确服务器地址'))
    }
  }

  /*
   * port验证
   * */
  public static Port(rule: any, value: any, callback: any) {
    const reg =
      /^([1-9]|[1-9]\d{1,3}|[1-5]\d{4}|6[0-4]\d{4}|65[0-4]\d|655[0-2]\d|6553[0-5])$/
    if (!value) {
      return callback(new Error('请输入正确端口,端口范围1-65535'))
    }
    if (!reg.test(value)) {
      return callback(new Error('请输入正确端口,端口范围1-65535'))
    }
    return callback()
  }

  /*
   * 整数正则
   * */
  public static checkInterNum(rule: any, value: any, callback: any) {
    const reg = /^[0-9]*[1-9][0-9]*$/
    if (!value) {
      return callback(new Error('请填写整数'))
    }
    if (!reg.test(value)) {
      return callback(new Error('请输入整数'))
    }
    return callback()
  }

  /*
   * URL正则
   * */
  public static isURL(rule: any, value: any, callback: any) {
    const reg = new RegExp(
      '^' +
        // protocol identifier
        '(?:(?:https?|ftp)://)' +
        // user:pass authentication
        '(?:\\S+(?::\\S*)?@)?' +
        '(?:' +
        // IP address dotted notation octets
        // excludes loopback network 0.0.0.0
        // excludes reserved space >= 224.0.0.0
        // excludes network & broacast addresses
        // (first & last IP address of each class)
        '(?:[1-9]\\d?|1\\d\\d|2[01]\\d|22[0-3])' +
        '(?:\\.(?:1?\\d{1,2}|2[0-4]\\d|25[0-5])){2}' +
        '(?:\\.(?:[1-9]\\d?|1\\d\\d|2[0-4]\\d|25[0-4]))' +
        '|' +
        // host name
        '(?:(?:[a-z\\u00a1-\\uffff0-9]-*)*[a-z\\u00a1-\\uffff0-9]+)' +
        // domain name
        '(?:\\.(?:[a-z\\u00a1-\\uffff0-9]-*)*[a-z\\u00a1-\\uffff0-9]+)*' +
        // TLD identifier
        '(?:\\.(?:[a-z\\u00a1-\\uffff]{2,}))' +
        ')' +
        // port number
        '(?::\\d{2,5})?' +
        // resource path
        '(?:/\\S*)?' +
        '$',
      'i'
    )
    if (!value) {
      return callback(new Error('请输入网页地址'))
    }
    if (!reg.test(value)) {
      return callback(new Error('网页地址不正确,例:<scheme>://<host>:<port>/<path>'))
    }
    return callback()
  }

  /*
   * 电话验证
   * */
  public static isPhone(rule: any, value: any, callback: any) {
    if (!value) {
      return callback(new Error('请输入电话号码'))
    }
    const pattern = /^(0\d{2,3}-\d{7,8})|(1[3456789]\d{9})$/
    if (pattern.test(value)) {
      return callback()
    }
    return callback(new Error('输入的电话号码错误'))
  }

  /*
   * 邮箱验证
   * */
  public static isEmail(rule: any, value: any, callback: any) {
    if (!value) {
      callback(new Error('请输入邮箱地址'))
      return false
    }
    const pattern = /^[A-Za-z0-9_-\u4e00-\u9fa5]+@[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+$/
    if (pattern.test(value)) {
      callback()
      return true
    }
    callback(new Error('请输入正确的邮箱地址'))
    return false
  }

  /*
   * 用户名
   * */
  public static isAccount(rule: any, value: any, callback: any) {
    if (value) {
      const pattern = /^[\u4E00-\u9FA5A-Za-z0-9_]+$/
      if (pattern.test(value)) {
        return callback()
      }
      return callback(
        new Error('不超过10个字符串，支持中文、大写字母、小写字母、数字、下划线 ')
      )
    }
    return callback()
  }

  /*
   * 用户姓名（角色配置、管理员配置使用），验证规则：中文、大写字母、小写字母、数字、下划线
   * */
  public static isLegalName(rule: any, value: any, callback: any) {
    if (!value) {
      callback(new Error(`请输入${rule.fieldName || '用户姓名'}`))
      return false
    }
    const pattern = /^[\u4E00-\u9FA5A-Za-z0-9_]+$/
    if (pattern.test(value)) {
      callback()
      return true
    }
    callback(new Error(`请输入正确的${rule.fieldName || '用户姓名'}`))
    return false
  }

  /*
   * 探针名称，验证规则：中文、大写字母、小写字母、数字、下划线、中划线
   * */
  public static isLegalTzName(rule: any, value: any, callback: any) {
    if (!value) {
      callback(new Error('请输入探针名称'))
      return false
    }
    const pattern = /^[\u4E00-\u9FA5A-Za-z0-9_-]+$/
    if (pattern.test(value)) {
      callback()
      return true
    }
    callback(new Error('请输入正确的探针名称'))
    return false
  }

  /**
   * 用户密码验证，验证规则：密码长度 8 到 32，并且包括大小写字母、数字和特殊字符四类
   */
  public static isLegalPwd(rule: any, value: any, callback: any, isSetting?: boolean) {
    if (!value) {
      callback(new Error('请输入密码'))
      return false
    }
    const pattern = /^[\x21-\x2f\x3a-\x40\x5b-\x60\x7B-\x7F-a-z_A-Z0-9]+$/
    // 若是设置-修改密码需要额外判断密码不能和账号相同
    if (isSetting && localStorage.getItem('account') === value) {
      callback(new Error('密码不能和账号相同'))
      return false
    }
    if (
      pattern.test(value) &&
      String(value).length <= 32 &&
      String(value).length >= 8
    ) {
      const codeStatus = Rules.checkStrong(value)
      if (codeStatus >= 3) {
        callback()
        return true
      }
      callback(
        new Error('密码强度不够，需要包括大小写字母、数字和特殊字符四类中的三类')
      )
      return false
    }
    callback(
      new Error(
        '密码长度在 8 到 32 个字符，并且包括大小写字母、数字和特殊字符四类中的三类'
      )
    )
    return false
  }

  /**
   * 用户密码强度校验
   */
  public static checkStrong(sValue: string) {
    let modes = 0
    if (sValue.length < 1) return modes
    if (/\d/.test(sValue)) modes += 1
    if (/[A-Z]/.test(sValue)) modes += 1
    if (/[a-z]/.test(sValue)) modes += 1
    if (/[\x21-\x2f\x3a-\x40\x5b-\x60\x7B-\x7F]/.test(sValue)) modes += 1
    return modes
  }
}
