import CryptoJS from 'crypto-js'
import Cookies from 'js-cookie'

export default class Util {
  /** ******
   * 获取以‘万’作为单位的数字
   * @param num Number
   * @returns {String}
   */
  public static getNumber(num: number) {
    if (num * 1 > 100000000) {
      return `${(num / 100000000).toFixed(2)}亿`
    }
    if (num * 1 > 10000) {
      return `${(num / 10000).toFixed(2)}万`
    }
    return num
  }

  /** ******
   * 转换流量
   * @param num Number
   * @returns {String}
   */
  public static handleSize(num: any, defaultUnit: number = 0) {
    if (num === 'noSuchInstance' || num === 'noSuchObject') {
      return '无数据'
    }
    const k = 1024
    const sizes = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'].slice(
      defaultUnit
    )
    if (num <= 0) return `0${sizes[0]}`
    const i = Math.floor(Math.log(num) / Math.log(k))
    // if (i < 2) i = 2
    return `${(num / k ** i).toFixed(2)}${sizes[i]}`
  }

  /**
   * 求百分比
   * @param num 当前数
   * @param total 总数
   * @returns {string}
   */
  public static GetPercent(num: number | string, total: number | string) {
    if (Number.isNaN(parseFloat(`${num}`)) || Number.isNaN(parseFloat(`${total}`))) {
      return '-'
    }
    return total <= 0
      ? '0%'
      : `${
          Math.round((parseFloat(`${num}`) / parseFloat(`${total}`)) * 10000) / 100.0
        }%`
  }

  public static decryptAES(word: string, chart: string) {
    const iv = CryptoJS.enc.Utf8.parse('0000000000000000')
    const key = CryptoJS.enc.Utf8.parse(
      `${chart}UGdfm24TH1fdsDHyUGdfm24TH51fdsDH`.slice(0, 32)
    )
    const decrypt = CryptoJS.AES.decrypt(word, key, {
      iv,
      mode: CryptoJS.mode.CBC,
      padding: CryptoJS.pad.Pkcs7
    })
    return CryptoJS.enc.Utf8.stringify(decrypt).toString()
  }

  public static encryptionAES(word: string, chart: string) {
    const key = CryptoJS.enc.Utf8.parse(
      `${chart}UGdfm24TH1fdsDHyUGdfm24TH51fdsDH`.slice(0, 32)
    )
    const iv = CryptoJS.enc.Utf8.parse('0000000000000000')
    const srcs = CryptoJS.enc.Utf8.parse(word)
    const encrypted = CryptoJS.AES.encrypt(srcs, key, {
      iv,
      mode: CryptoJS.mode.CBC,
      padding: CryptoJS.pad.Pkcs7
    })
    return encrypted.toString()
  }

  // token操作
  public static getToken() {
    return Cookies.get('Token')
  }

  public static setToken(token: string) {
    return Cookies.set('Token', token)
  }

  public static removeToken() {
    return Cookies.remove('Token')
  }

  // echarts适配
  public static getFontSize(num: number) {
    const clientWidth =
      window.innerWidth ||
      document.documentElement.clientWidth ||
      document.body.clientWidth
    if (!clientWidth) return ''
    return (num * clientWidth) / 1920
  }

  // 对Proxy对象进行拆箱，解决echarts的tooltip不显示问题
  public static unWarp(obj: any) {
    return obj && (obj.__v_raw || obj.valueOf() || obj)
  }

  public static FormatDate(paramdate: string | number | Date, paramfmt: string) {
    const date = new Date(paramdate)
    let fmt = paramfmt
    if (typeof fmt === 'undefined') {
      fmt = 'yyyy-MM-dd hh:mm:ss'
    }
    if (/(y+)/.test(fmt)) {
      fmt = fmt.replace(RegExp.$1, `${date.getFullYear()}`.substr(4 - RegExp.$1.length))
    }
    const o = {
      'M+': date.getMonth() + 1, // 月份
      'd+': date.getDate(), // 日
      'h+': date.getHours(), // 小时
      'm+': date.getMinutes(), // 分
      's+': date.getSeconds(), // 秒
      'q+': Math.floor((date.getMonth() + 3) / 3), // 季度
      S: date.getMilliseconds() // 毫秒
    }
    Object.keys(o).forEach((k) => {
      if (new RegExp(`(${k})`).test(fmt)) {
        const str = `${o[k]}`
        fmt = fmt.replace(
          RegExp.$1,
          RegExp.$1.length === 1 ? str : `00${str}`.substr(str.length)
        )
      }
    })

    return fmt
  }
}
